import 'package:flutter/widgets.dart';

abstract class Validator {
  final String errorText;
  List<Validator> _validators;

  Validator(this.errorText, List<Validator> children) : _validators = children;

  String? validate(String? value);

  @protected
  String? validateChildren(String? value) {
    for (final validator in _validators) {
      final error = validator.validate(value);
      if (error != null) {
        return error;
      }
    }
    return null;
  }

  static String? Function(String?) validateAll(List<Validator> validators) {
    return CompositeValidator(validators).validate;
  }
}

class CompositeValidator extends Validator {
  CompositeValidator(List<Validator> children) : super('', children);

  @override
  String? validate(String? value) {
    return validateChildren(value);
  }
}

class NotEmpty extends Validator {
  NotEmpty(String errorText) : super(errorText, []);

  @override
  String? validate(String? value) {
    return (value == null || value.isEmpty) ? errorText : null;
  }
}

final _emailRegexp = RegExp(r'^[a-zA-Z0-9.]+@[a-zA-Z0-9]+\.[a-zA-Z]+');

class EmailValidator extends Validator {
  EmailValidator(String errorText) : super(errorText, []);

  @override
  String? validate(String? value) {
    if (value == null) return errorText;
    return _emailRegexp.hasMatch(value) ? null : errorText;
  }
}

class ConfirmPasswordValidator extends Validator {
  final TextEditingController controller;

  ConfirmPasswordValidator(
    this.controller,
    String errorText,
  ) : super(errorText, []);

  @override
  String? validate(String? value) {
    return value == controller.text ? null : errorText;
  }
}

class PhoneValidator extends Validator {
  PhoneValidator(String errorText) : super(errorText, []);

  @override
  String? validate(String? value) {
    return value!.length < 7 ? errorText : null;
  }
}
